@@ -38,6 +38,7 @@ Specifically, execute is comprised of a series of steps that happen at
 different times during the loop.
 
 1. ``compute_time_context``
+---------------------------
 First, at the beginning of the main execution loop, ``compute_time_context`` is
 called. This function computes time contexts, and pass them to all children of
 the current node. These time contexts could be used in later steps to get data.
@@ -60,9 +61,9 @@ By default this function does nothing.
 -------------------
 
 Then, when an expression is ready to be evaluated we call
-:func:`~ibis.backends.pandas.core.execute` on the expressions arguments and then
-:func:`~ibis.backends.pandas.dispatch.execute_node` on the expression with its
-now-materialized arguments.
+:func:`~ibis.backends.pandas.core.execute` on the expressions arguments and
+then :func:`~ibis.backends.pandas.dispatch.execute_node` on the expression
+with its now-materialized arguments.
 
 4. ``post_execute``
 -------------------
@@ -88,7 +89,7 @@ passed to ``post_execute`` would be the bound values passed in at the time the
 
 
 Scope
--------------------
+-----
 Scope is used across the execution phases, it iss a map that maps Ibis
 operators to actual data. It is used to cache data for calculated ops. It is
 an optimization to reused executed results.
@@ -102,7 +103,6 @@ stored in value
 See ibis.common.scope for details about the implementaion.
 """
 
-from __future__ import absolute_import
 
 import datetime
 import functools
@@ -119,7 +119,7 @@ import ibis.expr.operations as ops
 import ibis.expr.types as ir
 import ibis.expr.window as win
 import ibis.util
-from ibis.client import find_backends
+from ibis.backends.base import BaseBackend
 from ibis.expr.scope import Scope
 from ibis.expr.timecontext import canonicalize_context
 from ibis.expr.typing import TimeContext
@@ -148,7 +148,7 @@ def is_computable_input(arg):
     return False
 
 
-@is_computable_input.register(ibis.client.Client)
+@is_computable_input.register(BaseBackend)
 @is_computable_input.register(ir.Expr)
 @is_computable_input.register(dt.DataType)
 @is_computable_input.register(type(None))
@@ -201,7 +201,7 @@ def execute_with_scope(
     # computing anything *and* before associating leaf nodes with data. This
     # allows clients to provide their own data for each leaf.
     if clients is None:
-        clients = list(find_backends(expr))
+        clients = expr._find_backends()
 
     if aggcontext is None:
         aggcontext = agg_ctx.Summarize()
@@ -255,7 +255,7 @@ def execute_until_in_scope(
     scope : Scope
     timecontext : Optional[TimeContext]
     aggcontext : Optional[AggregationContext]
-    clients : List[ibis.client.Client]
+    clients : List[ibis.backends.base.BaseBackend]
     kwargs : Mapping
     """
     # these should never be None
@@ -293,6 +293,7 @@ def execute_until_in_scope(
             num_args=len(computable_args),
             timecontext=timecontext,
             clients=clients,
+            scope=scope,
         )
     else:
         arg_timecontexts = [None] * len(computable_args)
@@ -341,7 +342,7 @@ def execute_until_in_scope(
     # if we're unable to find data then raise an exception
     if not scopes and computable_args:
         raise com.UnboundExpressionError(
-            'Unable to find data for expression:\n{}'.format(repr(expr))
+            f'Unable to find data for expression:\n{repr(expr)}'
         )
 
     # there should be exactly one dictionary per computable argument
@@ -429,7 +430,11 @@ def main_execute(
     params = {k.op() if hasattr(k, 'op') else k: v for k, v in params.items()}
     scope = scope.merge_scope(Scope(params, timecontext))
     return execute_with_scope(
-        expr, scope, timecontext=timecontext, aggcontext=aggcontext, **kwargs,
+        expr,
+        scope,
+        timecontext=timecontext,
+        aggcontext=aggcontext,
+        **kwargs,
     )
 
 
@@ -446,9 +451,9 @@ def execute_and_reset(
 
     Notes
     -----
-    The difference between this function and :func:`~ibis.backends.pandas.core.execute`
-    is that this function resets the index of the result, if the result has
-    an index.
+    The difference between this function and
+    :func:`~ibis.backends.pandas.core.execute` is that this function resets
+    the index of the result, if the result has an index.
 
     Parameters
     ----------
@@ -517,7 +522,7 @@ are going to be used in executeion and passes these attributes to children
 nodes.
 
 Param:
-clients: List[ibis.client.Client]
+clients: List[ibis.backends.base.BaseBackend]
     backends for execution
 timecontext : Optional[TimeContext]
     begin and end time context needed for execution
@@ -534,6 +539,9 @@ See ``computable_args`` in ``execute_until_in_scope``
 
 @compute_time_context.register(ops.Node)
 def compute_time_context_default(
-    node, timecontext: Optional[TimeContext] = None, **kwargs
+    node,
+    timecontext: Optional[TimeContext] = None,
+    scope: Optional[Scope] = None,
+    **kwargs,
 ):
     return [timecontext for arg in node.inputs if is_computable_input(arg)]
