@@ -1,8 +1,4 @@
-import pytest
-from pandas.testing import assert_frame_equal
-
-pytest.importorskip('pyspark')
-pytestmark = pytest.mark.pyspark
+import pandas.testing as tm
 
 
 def test_isnull(client):
@@ -19,7 +15,7 @@ def test_isnull(client):
         expected = table_pandas[table_pandas[col].isnull()].reset_index(
             drop=True
         )
-        assert_frame_equal(result, expected)
+        tm.assert_frame_equal(result, expected)
 
 
 def test_notnull(client):
@@ -36,4 +32,4 @@ def test_notnull(client):
         expected = table_pandas[table_pandas[col].notnull()].reset_index(
             drop=True
         )
-        assert_frame_equal(result, expected)
+        tm.assert_frame_equal(result, expected)
