@@ -1,5 +1,6 @@
 import itertools
 import os
+import warnings
 import webbrowser
 from typing import TYPE_CHECKING, Optional
 
@@ -8,7 +9,9 @@ import numpy as np
 import ibis
 import ibis.common.exceptions as com
 import ibis.config as config
+import ibis.expr.types as ir
 import ibis.util as util
+from ibis.config import options
 from ibis.expr.typing import TimeContext
 
 if TYPE_CHECKING:
@@ -38,8 +41,8 @@ class Expr:
         except com.TranslationError as e:
             output = (
                 'Translation to backend failed\n'
-                'Error message: {0}\n'
-                'Expression repr follows:\n{1}'.format(e.args[0], self._repr())
+                'Error message: {}\n'
+                'Expression repr follows:\n{}'.format(e.args[0], self._repr())
             )
             return output
         else:
@@ -50,7 +53,7 @@ class Expr:
 
     def __bool__(self):
         raise ValueError(
-            "The truth value of an Ibis expression is not " "defined"
+            "The truth value of an Ibis expression is not defined"
         )
 
     __nonzero__ = __bool__
@@ -123,7 +126,7 @@ class Expr:
         import ibis.expr.visualize as viz
 
         path = viz.draw(viz.to_graph(self), format=format)
-        webbrowser.open('file://{}'.format(os.path.abspath(path)))
+        webbrowser.open(f'file://{os.path.abspath(path)}')
 
     def pipe(self, f, *args, **kwargs):
         """Generic composition function to enable expression pipelining.
@@ -188,6 +191,55 @@ class Expr:
     def _factory(self):
         return type(self)
 
+    def _find_backends(self):
+        """Find possible backends for an expression.
+
+        Parameters
+        ----------
+        expr : Expr
+
+        Returns
+        -------
+        Backend
+            Backend found.
+        """
+        from ibis.backends.base import BaseBackend
+
+        seen_backends = set()
+
+        stack = [self.op()]
+        seen = set()
+
+        while stack:
+            node = stack.pop()
+
+            if node not in seen:
+                seen.add(node)
+
+                for arg in node.flat_args():
+                    if isinstance(arg, BaseBackend):
+                        seen_backends.add(arg)
+                    elif isinstance(arg, ir.Expr):
+                        stack.append(arg.op())
+
+        return list(seen_backends)
+
+    def _find_backend(self):
+        backends = self._find_backends()
+
+        if not backends:
+            default = options.default_backend
+            if default is None:
+                raise com.IbisError(
+                    'Expression depends on no backends, and found no default'
+                )
+            return default
+
+        if len(backends) > 1:
+            raise ValueError('Multiple backends found')
+
+        return backends[0]
+
     def execute(
         self,
         limit='default',
@@ -218,9 +270,7 @@ class Expr:
         result : expression-dependent
           Result of compiling expression and executing in backend
         """
-        from ibis.client import execute
-
-        return execute(
+        return self._find_backend().execute(
             self, limit=limit, timecontext=timecontext, params=params, **kwargs
         )
 
@@ -238,9 +288,7 @@ class Expr:
         compiled : value or list
            query representation or list thereof
         """
-        from ibis.client import compile
-
-        return compile(
+        return self._find_backend().compile(
             self, limit=limit, timecontext=timecontext, params=params
         )
 
@@ -248,6 +296,11 @@ class Expr:
         """
         Returns True if expression can be compiled to its attached client
         """
+        warnings.warn(
+            '`verify` is deprecated, use `compile` and capture the '
+            '`TranslationError` exception instead',
+            FutureWarning,
+        )
         try:
             self.compile()
         except Exception:
@@ -260,9 +313,6 @@ class Expr:
             return False
         return self._arg.equals(other._arg, cache=cache)
 
-    def _root_tables(self):
-        return self.op().root_tables()
-
 
 class ExprList(Expr):
     def _type_display(self):
@@ -371,7 +421,7 @@ class ScalarExpr(ValueExpr):
 
 class ColumnExpr(ValueExpr):
     def _type_display(self):
-        return '{}*'.format(self.type())
+        return f'{self.type()}*'
 
     def parent(self):
         return self._arg
@@ -380,7 +430,7 @@ class ColumnExpr(ValueExpr):
         """
         Promote this column expression to a table projection
         """
-        roots = self._root_tables()
+        roots = self.op().root_tables()
         if len(roots) > 1:
             raise com.RelationError(
                 'Cannot convert array expression '
@@ -407,6 +457,11 @@ class AnalyticExpr(Expr):
         return 'analytic'
 
 
+class ExistsExpr(AnalyticExpr):
+    def type(self):
+        return 'exists'
+
+
 class TableExpr(Expr):
     @property
     def _factory(self):
@@ -864,7 +919,7 @@ class StructColumn(AnyColumn, StructValue):
 
 
 class DestructValue(AnyValue):
-    """ Class that represents a destruct value.
+    """Class that represents a destruct value.
 
     When assigning a destruct column, the field inside this destruct column
     will be destructured and assigned to multipe columnns.
@@ -1217,7 +1272,7 @@ def literal(value, type=None):
             dtype = inferred_dtype.cast(explicit_dtype, value=value)
         except com.IbisTypeError:
             raise TypeError(
-                'Value {!r} cannot be safely coerced to {}'.format(value, type)
+                f'Value {value!r} cannot be safely coerced to {type}'
             )
     elif has_explicit:
         dtype = explicit_dtype
@@ -1267,34 +1322,59 @@ def as_value_expr(val):
     return val
 
 
-def param(type):
-    """Create a parameter of a particular type to be defined just before
-    execution.
+def array(values):
+    """Create an array expression.
+
+    If the input expressions are all column expressions, then the output will
+    be an ``ArrayColumn``. The input columns will be concatenated row-wise to
+    produce each array in the output array column. Each array will have length
+    n, where n is the number of input columns. All input columns should be of
+    the same datatype.
+
+    If the input expressions are Python literals, then the output will be a
+    single ``ArrayScalar`` of length n, where n are the number of input
+    values. This is equivalent to ``ibis.literal(values)``.
 
     Parameters
     ----------
-    type : dt.DataType
-        The type of the unbound parameter, e.g., double, int64, date, etc.
+    values : list
+        A list of Ibis column expressions, or a list of Python literals
 
     Returns
     -------
-    ScalarExpr
+    array_value : ArrayValue
+        An array column (if the inputs are column expressions), or an array
+        scalar (if the inputs are Python literals)
 
     Examples
     --------
+    Creating an array column from column expressions:
     >>> import ibis
-    >>> import ibis.expr.datatypes as dt
-    >>> start = ibis.param(dt.date)
-    >>> end = ibis.param(dt.date)
-    >>> schema = [('timestamp_col', 'timestamp'), ('value', 'double')]
-    >>> t = ibis.table(schema)
-    >>> predicates = [t.timestamp_col >= start, t.timestamp_col <= end]
-    >>> expr = t.filter(predicates).value.sum()
+    >>> t = ibis.table([('a', 'int64'), ('b', 'int64')], name='t')
+    >>> result = ibis.array([t.a, t.b])
+
+    Creating an array scalar from Python literals:
+    >>> import ibis
+    >>> result = ibis.array([1.0, 2.0, 3.0])
     """
-    import ibis.expr.datatypes as dt
     import ibis.expr.operations as ops
 
-    return ops.ScalarParameter(dt.dtype(type)).to_expr()
+    if all([isinstance(value, ColumnExpr) for value in values]):
+        return ops.ArrayColumn(values).to_expr()
+    elif any([isinstance(value, ColumnExpr) for value in values]):
+        raise com.IbisTypeError(
+            'To create an array column using `array`, all input values must '
+            'be column expressions.'
+        )
+    else:
+        try:
+            return literal(list(values))
+        except com.IbisTypeError as e:
+            raise com.IbisTypeError(
+                'Could not create an array scalar from the values provided '
+                'to `array`. Ensure that all input values have the same '
+                'Python type, or can be casted to a single Python type.'
+            ) from e
 
 
 class UnnamedMarker:
